from enum import Enum

class EmailProvider(str, Enum):
    SMTP = 'smtp'
    SENDGRID = 'sendgrid'

class SettingType(str, Enum):
    EMAIL = 'email'
    SECURITY = 'security'
    KYC = 'kyc'
    COMMISSION = 'commission'

class SettingStatus(str, Enum):
    ACTIVE = 'active'
    INACTIVE = 'inactive'
    PENDING = 'pending'

class ContentType(str, Enum):
    TEXT = 'text'
    HTML = 'html'

class BaseEmailVariables:
    """Base variables available for all system emails"""
    NAME = "name"
    EMAIL = "email"
    PHONE = "phone"
    ID = "id"
    FULL_NAME = "full_name"
    FIRST_NAME = "first_name"
    LAST_NAME = "last_name"
    LOCATION = "location"
    CP_URL = "cp_url"
    CRM_URL = "crm_url"
    USER_AGENT = "user_agent"
    IP_ADDRESS = "ip_address"
    TIMESTAMP = "timestamp"
    TYPE = "type"
    PASSWORD = "password"
    ERROR_TYPE = "error_type"
    DATE = "date"
    TIME = "time"
    CHANNEL = "channel"
    PASSWORD_RESET_LINK = "password_reset_link"

    CP_LOGIN_LINK = "cp_login_link"
    CRM_LOGIN_LINK = "crm_login_link"

    # Customer related
    CUSTOMER_NAME = "customer_name"
    CUSTOMER_EMAIL = "customer_email"
    CUSTOMER_PHONE = "customer_phone"
    CUSTOMER_ADDRESS = "customer_address"
    CUSTOMER_CITY = "customer_city"
    CUSTOMER_STATE = "customer_state"
    CUSTOMER_ZIP = "customer_zip"
    CUSTOMER_COUNTRY = "customer_country"
    CUSTOMER_ID = "customer_id"
    CUSTOMER_FIRST_NAME = "customer_first_name"
    CUSTOMER_LAST_NAME = "customer_last_name"
    CUSTOMER_CRM_LINK = "customer_crm_link"
    # User related
    USER_NAME = "user_name"  # Full name of the user
    USER_FULL_NAME = "user_full_name"  # Full name of the user
    USER_EMAIL = "user_email"  # Email address of the user
    USER_FIRST_NAME = "user_first_name"  # First name of the user
    USER_LAST_NAME = "user_last_name"  # Last name of the user
    # Company related
    COMPANY_NAME = "company_name"  # Name of the company
    COMPANY_ADDRESS = "company_address"  # Company address
    COMPANY_PHONE = "company_phone"  # Company phone number
    COMPANY_EMAIL = "company_email"  # Company email address
    COMPANY_WEBSITE = "company_website"  # Company website URL
    APP_NAME = "app_name"  # Name of the application
    APP_URL = "app_url"  # Base URL of the application
    CURRENT_DATE = "current_date"  # Current date
    CURRENT_TIME = "current_time"  # Current time
    SUPPORT_EMAIL = "support_email"  # Support email address
    LOGO_URL = "logo_url"  # URL to company logo

    # Common
    TICKET_ID = "ticket_id"  # Ticket ID
    TICKET_NUMBER = "ticket_number"  # Ticket number
    TICKET_URL = "ticket_url"  # Ticket URL
    TICKET_MESSAGE = "ticket_message"  # Ticket message
    TICKET_SUBJECT = "ticket_subject"  # Ticket subject
    TICKET_DESCRIPTION = "ticket_description"  # Ticket description
    TICKET_STATUS = "ticket_status"  # Ticket status
    TICKET_PRIORITY = "ticket_priority"  # Ticket priority
    TICKER_CATEGORY = "ticket_category"  # Ticket category

    # Referral
    REFERRAL_CODE = "referral_code"  # Referral code
    REFERRAL_LINK = "referral_link"  # Referral link
    REFERRAL_CAMPAIGN = "referral_campaign"  # Referral campaign
    REFERRAL_DASHBOARD_URL = "referral_dashboard_url"  # Referral dashboard URL

    FEEDBACK_LINK = "feedback_link"  # Feedback link
    VERIFICATION_LINK = "verification_link"  # Verification link
    EXPIRY_TIME = "expiry_time"  # Expiry time
    CODE = "code"  # Code
    REASON = "reason"  # Reason
    STATUS = "status"  # Status
    SEVERITY = "severity"  # Severity
    ACTION_REQUIRED = "action_required"  # Action required
    ACTION_URL = "action_url"  # Action URL

    # Specific Contextual URLs (often needed)
    MT5_DOWNLOAD_LINK = "mt5_download_link"
    SECURITY_SETTINGS_URL = "security_settings_url"
    PASSWORD_RESET_URL = "password_reset_url"
    ENTER_CODE_URL = "enter_code_url"
    REFERRAL_DASHBOARD_URL = "referral_dashboard_url"
    REWARD_HISTORY_URL = "reward_history_url"
    SUPPORT_URL = "support_url"
    TICKET_CRM_URL = "ticket_crm_url"
    REQUEST_HISTORY_URL = "request_history_url"
    IB_PORTAL_URL = "ib_portal_url"
    UPLOAD_DOCUMENT_URL = "upload_document_url"
    TRANSACTION_HISTORY_URL = "transaction_history_url"

    @classmethod
    def get_all_variables(cls) -> list:
        """Get all base variables as a list"""
        return [
            getattr(cls, attr) for attr in dir(cls)
            if not attr.startswith('_') and isinstance(getattr(cls, attr), str)
        ]

    @classmethod
    def get_variable_descriptions(cls) -> dict:
        """Get descriptions for all base variables"""
        return {
            cls.CP_URL: "URL to the CP",
            cls.CRM_URL: "URL to the CRM",
            cls.CRM_LOGIN_LINK: "Login link for the CRM",
            cls.CP_LOGIN_LINK: "Login link for the CP",
            cls.NAME: "Full name of the receiver (customer or user, can be used when both details are not required)",
            cls.EMAIL: "Email address of the receiver (customer or user, can be used when both details are not required)",
            cls.PHONE: "Phone number of the receiver (customer or user, can be used when both details are not required)",
            cls.ID: "ID of the receiver (customer or user, can be used when both details are not required)",
            cls.FIRST_NAME: "First name of the receiver (customer or user, can be used when both details are not required)",
            cls.LAST_NAME: "Last name of the receiver (customer or user, can be used when both details are not required)",
            cls.LOCATION: "Location of the receiver (customer or user, can be used when both details are not required)",
            cls.CUSTOMER_NAME: "Full name of the customer",
            cls.CUSTOMER_ID: "ID of the customer",
            cls.CUSTOMER_EMAIL: "Email address of the customer",
            cls.CUSTOMER_PHONE: "Phone number of the customer",
            cls.CUSTOMER_ADDRESS: "Address of the customer",
            cls.CUSTOMER_CITY: "City of the customer",
            cls.CUSTOMER_STATE: "State of the customer",
            cls.CUSTOMER_ZIP: "Zip code of the customer",
            cls.CUSTOMER_COUNTRY: "Country of the customer",
            cls.CUSTOMER_FIRST_NAME: "First name of the customer",
            cls.CUSTOMER_LAST_NAME: "Last name of the customer",
            cls.USER_NAME: "Full name of the user",
            cls.USER_EMAIL: "Email address of the user",
            cls.USER_FIRST_NAME: "First name of the user",
            cls.USER_LAST_NAME: "Last name of the user",
            cls.COMPANY_NAME: "Name of the company",
            cls.COMPANY_ADDRESS: "Company address",
            cls.COMPANY_PHONE: "Company phone number",
            cls.COMPANY_EMAIL: "Company email address",
            cls.COMPANY_WEBSITE: "Company website URL",
            cls.APP_NAME: "Name of the application",
            cls.APP_URL: "Base URL of the application",
            cls.CURRENT_DATE: "Current date",
            cls.CURRENT_TIME: "Current time",
            cls.SUPPORT_EMAIL: "Support email address",
            cls.LOGO_URL: "URL to company logo",
            cls.TICKET_NUMBER: "Ticket number",
            cls.TICKET_URL: "Ticket URL",
            cls.FEEDBACK_LINK: "Feedback link",
            cls.VERIFICATION_LINK: "Verification link",
            cls.EXPIRY_TIME: "Expiry time",
            cls.CODE: "Code",
            cls.REASON: "Reason",
            cls.STATUS: "Status",
            cls.SEVERITY: "Severity level of the alert",
            cls.ACTION_REQUIRED: "Required action to be taken",
            cls.ACTION_URL: "Generic Action URL",
            cls.MT5_DOWNLOAD_LINK: "Link to download the MT5 platform",
            cls.SECURITY_SETTINGS_URL: "Link to user's security settings page",
            cls.PASSWORD_RESET_URL: "Link to the password reset page",
            cls.ENTER_CODE_URL: "Link to the page where verification code is entered",
            cls.REFERRAL_DASHBOARD_URL: "Link to the referrer's dashboard/tracking page",
            cls.REWARD_HISTORY_URL: "Link to the reward/payment history page",
            cls.SUPPORT_URL: "Link to the main support/contact page",
            cls.TICKET_CRM_URL: "Link for CRM users to view a specific ticket",
            cls.REQUEST_HISTORY_URL: "Link to the user's request history page",
            cls.IB_PORTAL_URL: "Link to the IB portal section",
            cls.UPLOAD_DOCUMENT_URL: "Link to the document upload page",
            cls.TRANSACTION_HISTORY_URL: "Link to the transaction history page",
            cls.TICKET_ID: "ID of the ticket",
            cls.TICKER_CATEGORY: "Category of the ticket",
            cls.TICKET_PRIORITY: "Priority of the ticket",
            cls.TICKET_STATUS: "Status of the ticket",
            cls.TICKET_SUBJECT: "Subject of the ticket",
            cls.TICKET_DESCRIPTION: "Description of the ticket",
            cls.TICKET_MESSAGE: "Message of the ticket",
            cls.REFERRAL_CODE: "Referral code of the referrer",
            cls.REFERRAL_LINK: "Link to the referrer's dashboard/tracking page",
            cls.REFERRAL_CAMPAIGN: "Campaign of the referrer",
            cls.REFERRAL_DASHBOARD_URL: "Link to the referrer's dashboard/tracking page",

        } 

class SystemEmailTrigger:
    """System email trigger events and their descriptions"""
    
    # Authentication related
    LOGIN_NOTIFICATION = "login_notification"
    LOGIN_FAILED_NOTIFICATION = "login_failed_notification"
    PASSWORD_CHANGED = "password_changed"
    PASSWORD_RESET = "password_reset"
    PASSWORD_RESET_REQUEST = "password_reset_request"
    EMAIL_VERIFICATION = "email_verification"
    EMAIL_VERIFICATION_FAILED = "email_verification_failed"
    EMAIL_VERIFICATION_SUCCESS = "email_verification_success"
    TWO_FACTOR_CODE = "two_factor_code"
    ACCESS_CHANGED = "access_changed"
    TWO_FACTOR_STATUS_CHANGED = "two_factor_status_changed"
    SECURITY_KEY_STATUS_CHANGED = "security_key_status_changed"
    
    # Customer related
    CUSTOMER_WELCOME_LIVE_EMAIL = "customer_welcome_live_email"
    CUSTOMER_WELCOME_DEMO_EMAIL = "customer_welcome_demo_email"
    CUSTOMER_WELCOME_IB_EMAIL = "customer_welcome_ib_email"
    CUSTOMER_AGENT_ASSIGNED = "customer_agent_assigned"
    CUSTOMER_AGENT_UNASSIGNED = "customer_agent_unassigned"
    CUSTOMER_FEEDBACK = "customer_feedback"
    CUSTOMER_SUPPORT = "customer_support"

    # User related
    USER_WELCOME_EMAIL = "user_welcome_email"
    USER_CUSTOMER_ASSIGNED = "user_customer_assigned"
    USER_CUSTOMER_UNASSIGNED = "user_customer_unassigned"
    USER_CUSTOMER_ADDED = "user_customer_added"

    # System related
    SYSTEM_ALERT = "system_alert"
    MAINTENANCE_NOTIFICATION = "maintenance_notification"
    SECURITY_ALERT = "security_alert"

    # Accounts related 
    ACCOUNT_UPDATED = "account_updated"
    ACCOUNT_CREATED = "account_created"
    ACCOUNT_LINKED = "account_linked"
    ACCOUNT_STATUS_CHANGED = "account_status_changed"
    ACCOUNT_TRADING_STATUS_CHANGED = "account_trading_status_changed"
    ACCOUNT_PASSWORD_CHANGED = "account_password_changed"
    ACCOUNT_RESET_PASSWORD = "account_reset_password"
    ACCOUNT_LEVERAGE_CHANGED = "account_leverage_changed"
    ACCOUNT_GROUP_CHANGED = "account_group_changed"
    ACCOUNT_ARCHIVED = "account_archived"
    ACCOUNT_RESTORED = "account_restored"

    # Documents Related
    DOCUMENT_UPLOADED = "document_uploaded"
    DOCUMENT_STATUS_CHANGED = "document_status_changed"
    DOCUMENT_VERIFICATION_REQUIRED = "document_verification_required"

    # KYC Related
    KYC_SUBMITTED = "kyc_submitted"
    KYC_APPROVED = "kyc_approved"
    KYC_REJECTED = "kyc_rejected"
    KYC_RESUBMITTED = "kyc_resubmitted"
    KYC_ADDITIONAL_VERIFICATION_REQUIRED = "kyc_additional_verification_required"

    # Transactions Related
    TRANSACTION_CREATED = "transaction_created"
    TRANSACTION_STATUS_CHANGED = "transaction_status_changed"
    TRANSACTION_FAILED = "transaction_failed"
    TRANSACTION_APPROVED = "transaction_approved"
    TRANSACTION_SUCCESS = "transaction_success"
    TRANSACTION_REJECTED = "transaction_rejected"
    TRANSACTION_EVIDENCE_REQUIRED = "transaction_evidence_required"
    TRANSACTION_EVIDENCE_UPLOADED = "transaction_evidence_uploaded"
    TRANSACTION_EVIDENCE_VERIFIED = "transaction_evidence_verified"
    TRANSACTION_EVIDENCE_REJECTED = "transaction_evidence_rejected"

    # Referral Related
    REFERRAL_CREATED = "referral_created"
    REFERRAL_AQUISITION = "referral_acquisition"
    REFERRAL_EVENT = "referral_event"
    REFERRAL_REWARD_EARNED = "referral_reward_earned"
    REFERRAL_REWARD_PAYMENT_FAILED = "referral_reward_payment_failed"
    REFERRAL_REWARD_PAYMENT_SUCCESS = "referral_reward_payment_success"

    # Ticket Related
    TICKET_CREATED = "ticket_created"
    TICKET_STATUS_CHANGED = "ticket_status_changed"
    TICKET_PRIORITY_CHANGED = "ticket_priority_changed"
    TICKET_ASSIGNED = "ticket_assigned"
    TICKET_NEW_MESSAGE = "ticket_new_message"
    TICKET_CLOSED = "ticket_closed"

    # Requests Related
    REQUEST_CREATED = "request_created"
    REQUEST_STATUS_CHANGED = "request_status_changed"
    ACCOUNT_REQUEST_APPROVED = "account_request_approved"
    LEVERAGE_REQUEST_APPROVED = "leverage_request_approved"
    IB_REQUEST_APPROVED = "ib_request_approved"
    REQUEST_REJECTED = "request_rejected"

    REFERRAL_LINK = "referral_link"
    REFERRAL_CURRENCY = "referral_currency"

    @classmethod
    def get_all_triggers(cls) -> list:
        """Get all trigger events as a list"""
        return [
            getattr(cls, attr) for attr in dir(cls)
            if not attr.startswith('_') and isinstance(getattr(cls, attr), str)
        ]

    @classmethod
    def get_trigger_descriptions(cls) -> dict:
        """Get descriptions for all trigger events"""
        return {
            cls.LOGIN_NOTIFICATION: "Sent when a user/customer logs in",
            cls.LOGIN_FAILED_NOTIFICATION: "Sent when a user's/customer's login attempt fails",
            cls.PASSWORD_CHANGED: "Sent when a user/customer changes their password",
            cls.PASSWORD_RESET: "Sent when a user/customer requests a password reset",
            cls.PASSWORD_RESET_REQUEST: "Sent when a user/customer requests a password reset",
            cls.EMAIL_VERIFICATION: "Sent to verify a user's/customer's email address",
            cls.EMAIL_VERIFICATION_FAILED: "Sent when a user's/customer's email verification fails",
            cls.EMAIL_VERIFICATION_SUCCESS: "Sent when a user's/customer's email verification succeeds",
            cls.TWO_FACTOR_CODE: "Sent with two-factor authentication code",
            cls.ACCESS_CHANGED: "Sent when a user's/customer's access level is changed",
            cls.TWO_FACTOR_STATUS_CHANGED: "Sent when two-factor authentication status is changed",
            cls.SECURITY_KEY_STATUS_CHANGED: "Sent when security key status is changed",
            
            cls.CUSTOMER_WELCOME_LIVE_EMAIL: "Sent when a customer registers via /live or added as a client in the crm",
            cls.CUSTOMER_WELCOME_DEMO_EMAIL: "Sent when a customer registers via /demo or added as a lead in the crm",
            cls.CUSTOMER_WELCOME_IB_EMAIL: "Sent when a customer registers via /ib or added as a ib in the crm",
            cls.CUSTOMER_AGENT_ASSIGNED: "Sent when a customer is assigned to an agent",
            cls.CUSTOMER_AGENT_UNASSIGNED: "Sent when a customer is unassigned from an agent",
            cls.CUSTOMER_FEEDBACK: "Sent to request customer feedback",
            cls.CUSTOMER_SUPPORT: "Sent in response to customer support requests",
            cls.USER_WELCOME_EMAIL: "Sent when a new user registers/added to the system",
            cls.USER_CUSTOMER_ASSIGNED: "Sent when a user is assigned to a customer",
            cls.USER_CUSTOMER_UNASSIGNED: "Sent when a user is unassigned from a customer",
            cls.USER_CUSTOMER_ADDED: "Sent when a user is assigned to a customer",
            
            cls.ACCOUNT_UPDATED: "Sent when account details are updated",
            cls.ACCOUNT_CREATED: "Sent when a new account is created",
            cls.ACCOUNT_LINKED: "Sent when a new account is linked to a customer",
            cls.ACCOUNT_STATUS_CHANGED: "Sent when an account status is changed",
            cls.ACCOUNT_TRADING_STATUS_CHANGED: "Sent when an account trading status is changed",
            cls.ACCOUNT_PASSWORD_CHANGED: "Sent when an account password is changed",
            cls.ACCOUNT_RESET_PASSWORD: "Sent when an account password is reset",
            cls.ACCOUNT_LEVERAGE_CHANGED: "Sent when an account leverage is changed",
            cls.ACCOUNT_GROUP_CHANGED: "Sent when an account group is changed",
            cls.ACCOUNT_ARCHIVED: "Sent when an account is archived",
            cls.ACCOUNT_RESTORED: "Sent when an account is restored",

            cls.DOCUMENT_UPLOADED: "Sent when a document is uploaded",
            cls.DOCUMENT_STATUS_CHANGED: "Sent when a document status is changed",
            cls.DOCUMENT_VERIFICATION_REQUIRED: "Sent when a document verification is required",

            cls.KYC_SUBMITTED: "Sent when a kyc is submitted",
            cls.KYC_APPROVED: "Sent when a kyc is approved",
            cls.KYC_REJECTED: "Sent when a kyc is rejected",
            cls.KYC_RESUBMITTED: "Sent when a kyc is resubmitted",

            cls.TRANSACTION_CREATED: "Sent when a transaction is created",
            cls.TRANSACTION_STATUS_CHANGED: "Sent when a transaction status is changed",
            cls.TRANSACTION_FAILED: "Sent when a transaction fails",
            cls.TRANSACTION_SUCCESS: "Sent when a transaction is successful",
            cls.TRANSACTION_REJECTED: "Sent when a transaction is rejected",
            cls.TRANSACTION_EVIDENCE_REQUIRED: "Sent when a transaction evidence is required",
            cls.TRANSACTION_EVIDENCE_UPLOADED: "Sent when a transaction evidence is uploaded",
            cls.TRANSACTION_EVIDENCE_VERIFIED: "Sent when a transaction evidence is verified",
            cls.TRANSACTION_EVIDENCE_REJECTED: "Sent when a transaction evidence is rejected",

            cls.REFERRAL_CREATED: "Sent when a referral is created",
            cls.REFERRAL_AQUISITION: "Sent when a referral is acquired",
            cls.REFERRAL_EVENT: "Sent when a referral event occurs",
            cls.REFERRAL_REWARD_EARNED: "Sent when a referral reward is earned",
            cls.REFERRAL_REWARD_PAYMENT_FAILED: "Sent when a referral reward payment fails",
            cls.REFERRAL_REWARD_PAYMENT_SUCCESS: "Sent when a referral reward payment is successful",

            cls.TICKET_CREATED: "Sent when a ticket is created",
            cls.TICKET_STATUS_CHANGED: "Sent when a ticket status is changed",
            cls.TICKET_PRIORITY_CHANGED: "Sent when a ticket priority is changed",
            cls.TICKET_ASSIGNED: "Sent when a ticket is assigned to an agent",
            cls.TICKET_NEW_MESSAGE: "Sent when a new message is added to a ticket",
            cls.TICKET_CLOSED: "Sent when a ticket is closed",

            cls.REQUEST_CREATED: "Sent when a request is created",
            cls.REQUEST_STATUS_CHANGED: "Sent when a request status is changed",
            cls.ACCOUNT_REQUEST_APPROVED: "Sent when an account request is approved",
            cls.LEVERAGE_REQUEST_APPROVED: "Sent when a leverage request is approved",
            cls.IB_REQUEST_APPROVED: "Sent when an ib request is approved",
            cls.REQUEST_REJECTED: "Sent when a request is rejected",

            cls.SYSTEM_ALERT: "Sent for important system alerts",
            cls.MAINTENANCE_NOTIFICATION: "Sent for scheduled maintenance notifications",
            cls.SECURITY_ALERT: "Sent for security-related alerts"
        }

    @classmethod
    def get_trigger_variables(cls) -> dict:
        """Get required variables for each trigger event"""
        return {
            cls.LOGIN_NOTIFICATION: ["name", "ip", "location"],
            cls.LOGIN_FAILED_NOTIFICATION: ["name", "ip", "location"],
            cls.PASSWORD_RESET: ["name", "password"],
            cls.PASSWORD_RESET_REQUEST: ["name", "verification_link", "expiry_time"],
            cls.PASSWORD_CHANGED: ["name"],
            cls.EMAIL_VERIFICATION: ["name", "code", "expiry_time"],
            cls.EMAIL_VERIFICATION_FAILED: ["name", "code", "expiry_time"],
            cls.EMAIL_VERIFICATION_SUCCESS: ["name", "code", "expiry_time"],
            cls.TWO_FACTOR_CODE: ["name", "code", "expiry_time"],
            cls.ACCESS_CHANGED: ["name", "reason", "support_contact", "status"],
            cls.TWO_FACTOR_STATUS_CHANGED: ["name", "status"],
            cls.SECURITY_KEY_STATUS_CHANGED: ["name", "status"],
            
            cls.CUSTOMER_WELCOME_LIVE_EMAIL: ["name", "cp_login_link"],
            cls.CUSTOMER_WELCOME_DEMO_EMAIL: ["name", "cp_login_link"],
            cls.CUSTOMER_WELCOME_IB_EMAIL: ["name", "cp_login_link"],
            cls.CUSTOMER_AGENT_ASSIGNED: ["customer_name", "user_name", "cp_login_link"],
            cls.CUSTOMER_AGENT_UNASSIGNED: ["customer_name", "user_name", "cp_login_link"],
            cls.CUSTOMER_FEEDBACK: ["user_name", "feedback_link"],
            cls.CUSTOMER_SUPPORT: ["ticket_number", "support_message"],

            cls.ACCOUNT_UPDATED: ["user_name", "updated_fields"],
            cls.ACCOUNT_CREATED: ["user_name", "account_number", "account_type", "server", "group_name", "leverage"],
            cls.ACCOUNT_LINKED: ["user_name", "account_number", "account_type", "server", "group_name", "leverage"],

            cls.DOCUMENT_UPLOADED: ["user_name", "document_name", "document_type", "document_status"],
            cls.DOCUMENT_STATUS_CHANGED: ["user_name", "document_name", "document_type", "document_status"],
            cls.DOCUMENT_VERIFICATION_REQUIRED: ["user_name", "document_name", "document_type", "document_status"],

            cls.KYC_SUBMITTED: ["user_name", "kyc_type", "kyc_status"],
            cls.KYC_APPROVED: ["user_name", "kyc_type", "kyc_status"],
            cls.KYC_REJECTED: ["user_name", "kyc_type", "kyc_status"],
            cls.KYC_RESUBMITTED: ["user_name", "kyc_type", "kyc_status"],

            cls.TRANSACTION_CREATED: ["user_name", "transaction_type", "transaction_status"],
            cls.TRANSACTION_STATUS_CHANGED: ["user_name", "transaction_type", "transaction_status"],
            cls.TRANSACTION_FAILED: ["user_name", "transaction_type", "transaction_status"],
            cls.TRANSACTION_SUCCESS: ["user_name", "transaction_type", "transaction_status"],
            cls.TRANSACTION_REJECTED: ["user_name", "transaction_type", "transaction_status"],
            cls.TRANSACTION_EVIDENCE_REQUIRED: ["user_name", "transaction_type", "transaction_status"],
            cls.TRANSACTION_EVIDENCE_UPLOADED: ["user_name", "transaction_type", "transaction_status"],
            cls.TRANSACTION_EVIDENCE_VERIFIED: ["user_name", "transaction_type", "transaction_status"],
            cls.TRANSACTION_EVIDENCE_REJECTED: ["user_name", "transaction_type", "transaction_status"],

            cls.REFERRAL_CREATED: ["user_name", "referral_type", "referral_status"],
            cls.REFERRAL_AQUISITION: ["user_name", "referral_type", "referral_status"],
            cls.REFERRAL_EVENT: ["user_name", "referral_type", "referral_status"],
            cls.REFERRAL_REWARD_EARNED: ["user_name", "referral_type", "referral_status"],
            cls.REFERRAL_REWARD_PAYMENT_FAILED: ["user_name", "referral_type", "referral_status"],
            cls.REFERRAL_REWARD_PAYMENT_SUCCESS: ["user_name", "referral_type", "referral_status"],

            cls.TICKET_CREATED: ["user_name", "ticket_type", "ticket_status"],
            cls.TICKET_STATUS_CHANGED: ["user_name", "ticket_type", "ticket_status"],
            cls.TICKET_PRIORITY_CHANGED: ["user_name", "ticket_type", "ticket_status"],
            cls.TICKET_ASSIGNED: ["user_name", "ticket_type", "ticket_status"],
            cls.TICKET_NEW_MESSAGE: ["user_name", "ticket_type", "ticket_status"],
            cls.TICKET_CLOSED: ["user_name", "ticket_type", "ticket_status"],

            cls.REQUEST_CREATED: ["user_name", "request_type", "request_status"],
            cls.REQUEST_STATUS_CHANGED: ["user_name", "request_type", "request_status"],
            cls.ACCOUNT_REQUEST_APPROVED: ["user_name", "request_type", "request_status"],
            cls.LEVERAGE_REQUEST_APPROVED: ["user_name", "request_type", "request_status"],
            cls.IB_REQUEST_APPROVED: ["user_name", "request_type", "request_status"],
            cls.REQUEST_REJECTED: ["user_name", "request_type", "request_status"],

            cls.USER_WELCOME_EMAIL: ["name", "crm_login_link"],
            cls.USER_CUSTOMER_ASSIGNED: ["customer_name", "user_name", "crm_login_link", "customer_crm_link"],
            cls.USER_CUSTOMER_UNASSIGNED: ["customer_name", "user_name", "crm_login_link", "customer_crm_link"],
            cls.USER_CUSTOMER_ADDED: ["customer_name", "user_name", "crm_login_link", "customer_crm_link"],
            
            cls.SYSTEM_ALERT: ["alert_message", "severity", "action_required"],
            cls.MAINTENANCE_NOTIFICATION: ["start_time", "end_time", "affected_services"],
            cls.SECURITY_ALERT: ["alert_type", "alert_message", "recommended_action"]
        }

# Variable classes for different types of emails
class AuthenticationEmailVariables:
    """Variables specific to authentication-related emails"""
    IP_ADDRESS = "ip_address"
    BROWSER = "browser"
    DEVICE = "device"
    LOCATION = "location"
    VERIFICATION_CODE = "verification_code"
    VERIFICATION_LINK = "verification_link"
    EXPIRY_TIME = "expiry_time"
    ACCESS_LEVEL = "access_level"
    PREVIOUS_ACCESS = "previous_access"
    NEW_ACCESS = "new_access"
    SECURITY_STATUS = "security_status"

    @classmethod
    def get_all_variables(cls) -> list:
        return [
            getattr(cls, attr) for attr in dir(cls)
            if not attr.startswith('_') and isinstance(getattr(cls, attr), str)
        ]

    @classmethod
    def get_variable_descriptions(cls) -> dict:
        return {
            cls.IP_ADDRESS: "IP address of the login attempt",
            cls.BROWSER: "Browser used for the login attempt",
            cls.DEVICE: "Device used for the login attempt",
            cls.LOCATION: "Location of the login attempt",
            cls.VERIFICATION_CODE: "Code for email verification or 2FA",
            cls.VERIFICATION_LINK: "Link for email verification or password reset",
            cls.EXPIRY_TIME: "Expiry time for verification code/link",
            cls.ACCESS_LEVEL: "User's access level",
            cls.PREVIOUS_ACCESS: "Previous access level",
            cls.NEW_ACCESS: "New access level",
            cls.SECURITY_STATUS: "Status of security feature"
        }

class CustomerEmailVariables:
    """Variables specific to customer-related emails"""
    ACCOUNT_TYPE = "account_type"
    ACCOUNT_NUMBER = "account_number"
    ACCOUNT_STATUS = "account_status"
    ASSIGNED_AGENT = "assigned_agent"
    PREVIOUS_AGENT = "previous_agent"
    FEEDBACK_LINK = "feedback_link"
    TICKET_NUMBER = "ticket_number"
    TICKET_STATUS = "ticket_status"
    TICKET_PRIORITY = "ticket_priority"
    SUPPORT_MESSAGE = "support_message"
    UPDATED_FIELDS = "updated_fields"

    @classmethod
    def get_all_variables(cls) -> list:
        return [
            getattr(cls, attr) for attr in dir(cls)
            if not attr.startswith('_') and isinstance(getattr(cls, attr), str)
        ]

    @classmethod
    def get_variable_descriptions(cls) -> dict:
        return {
            cls.ACCOUNT_TYPE: "Type of customer account (live/demo/ib)",
            cls.ACCOUNT_NUMBER: "Customer's account number",
            cls.ACCOUNT_STATUS: "Status of customer's account",
            cls.ASSIGNED_AGENT: "Name of assigned agent",
            cls.PREVIOUS_AGENT: "Name of previously assigned agent",
            cls.FEEDBACK_LINK: "Link to provide feedback",
            cls.TICKET_NUMBER: "Support ticket number",
            cls.TICKET_STATUS: "Status of support ticket",
            cls.TICKET_PRIORITY: "Priority of support ticket",
            cls.SUPPORT_MESSAGE: "Support message or response",
            cls.UPDATED_FIELDS: "List of updated account fields"
        }
    

class AccountEmailVariables:
    """Variables specific to account-related emails"""
    LOGIN = "login"
    MASTER_PASSWORD = "master_password"
    INVESTOR_PASSWORD = "investor_password"
    ACCOUNT_TYPE = "account_type"
    SERVER = "server"
    GROUP_NAME = "group_name"
    GROUP = "group"
    LEVERAGE = "leverage"
    STATUS = "status"

    @classmethod
    def get_all_variables(cls) -> list:
        return [
            getattr(cls, attr) for attr in dir(cls)
            if not attr.startswith('_') and isinstance(getattr(cls, attr), str)
        ]
    
    @classmethod
    def get_variable_descriptions(cls) -> dict:
        return {
            cls.LOGIN: "Login ID of the account",
            cls.MASTER_PASSWORD: "Master password of the account",
            cls.INVESTOR_PASSWORD: "Investor password of the account",
            cls.ACCOUNT_TYPE: "Type of account (live/demo/ib)",
            cls.SERVER: "Server of the account",
            cls.GROUP_NAME: "Group name of the account",
            cls.GROUP: "Group of the account",
            cls.LEVERAGE: "Leverage of the account",
            cls.STATUS: "Status of the account"
        }
    
class DocumentEmailVariables:
    """Variables specific to document-related emails"""
    DOCUMENT_NAME = "document_name"
    DOCUMENT_TYPE = "document_type"
    DOCUMENT_STATUS = "document_status"
    DOCUMENT_MESSAGE = "document_message"
    
    @classmethod
    def get_all_variables(cls) -> list:
        return [
            getattr(cls, attr) for attr in dir(cls)
            if not attr.startswith('_') and isinstance(getattr(cls, attr), str)
        ]
    
    @classmethod
    def get_variable_descriptions(cls) -> dict:
        return {
            cls.DOCUMENT_NAME: "Name of the document",
            cls.DOCUMENT_TYPE: "Type of the document",
            cls.DOCUMENT_STATUS: "Status of the document",
            cls.DOCUMENT_MESSAGE: "Message of the document"
        }
    

class KYCEmailVariables:
    """Variables specific to KYC-related emails"""
    KYC_TYPE = "kyc_type"
    KYC_STATUS = "kyc_status"
    KYC_MESSAGE = "kyc_message"
    KYC_DOCUMENT_TYPE = "kyc_document_type"
    
    @classmethod
    def get_all_variables(cls) -> list:
        return [
            getattr(cls, attr) for attr in dir(cls)
            if not attr.startswith('_') and isinstance(getattr(cls, attr), str)
        ]
    
    @classmethod
    def get_variable_descriptions(cls) -> dict:
        return {
            cls.KYC_TYPE: "Type of KYC",
            cls.KYC_STATUS: "Status of KYC",
            cls.KYC_MESSAGE: "Message of KYC",
            cls.KYC_DOCUMENT_TYPE: "Type of KYC document"
        }
    
class TransactionEmailVariables:
    """Variables specific to transaction-related emails"""
    TRANSACTION_TYPE = "transaction_type"
    TRANSACTION_STATUS = "transaction_status"
    TRANSACTION_MESSAGE = "transaction_message"
    TRANSACTION_DIRECTION = "transaction_direction"
    TRANSACTION_MT5_TICKET = "transaction_mt5_ticket"
    TRANSACTION_ACCOUNT_NUMBER = "transaction_account_number"
    TRANSACTION_ID = "transaction_id"
    TRANSACTION_AMOUNT = "transaction_amount"
    TRANSACTION_CURRENCY = "transaction_currency"
    TRANSACTION_DATE = "transaction_date"
    TRANSACTION_TIME = "transaction_time"
    TRANSACTION_FEE = "transaction_fee"
    TRANSACTION_PAYMENT_METHOD = "transaction_payment_method"
    TRANSACTION_PAYMENT_GATEWAY = "transaction_payment_gateway"
    TRANSACTION_PAYMENT_GATEWAY_ID = "transaction_payment_gateway_id"
    TRANSACTION_PAYMENT_GATEWAY_STATUS = "transaction_payment_gateway_status"
    TRANSACTION_PAYMENT_GATEWAY_MESSAGE = "transaction_payment_gateway_message"
    TRANSACTION_PAYMENT_GATEWAY_ERROR = "transaction_payment_gateway_error"
    TRANSACTION_PAYMENT_GATEWAY_ERROR_MESSAGE = "transaction_payment_gateway_error_message"
    TRANSACTION_PAYMENT_GATEWAY_ERROR_CODE = "transaction_payment_gateway_error_code"
    TRANSACTION_PAYMENT_GATEWAY_ERROR_DETAILS = "transaction_payment_gateway_error_details"
    TRANSACTION_REJECT_REASON = "transaction_reject_reason"
    
    @classmethod
    def get_all_variables(cls) -> list:
        return [
            getattr(cls, attr) for attr in dir(cls)
            if not attr.startswith('_') and isinstance(getattr(cls, attr), str)
        ]
    
    @classmethod
    def get_variable_descriptions(cls) -> dict:
        return {
            cls.TRANSACTION_TYPE: "Type of transaction",
            cls.TRANSACTION_STATUS: "Status of transaction",
            cls.TRANSACTION_MESSAGE: "Message of transaction",
            cls.TRANSACTION_ID: "ID of transaction",
            cls.TRANSACTION_AMOUNT: "Amount of transaction",
            cls.TRANSACTION_CURRENCY: "Currency of transaction",
            cls.TRANSACTION_DATE: "Date of transaction",
            cls.TRANSACTION_TIME: "Time of transaction",
            cls.TRANSACTION_FEE: "Fee of transaction",
            cls.TRANSACTION_DIRECTION: "Direction of transaction",
            cls.TRANSACTION_MT5_TICKET: "MT5 ticket of transaction",
            cls.TRANSACTION_ACCOUNT_NUMBER: "Account number of transaction",
            cls.TRANSACTION_PAYMENT_METHOD: "Payment method of transaction",
            cls.TRANSACTION_PAYMENT_GATEWAY: "Payment gateway of transaction",
            cls.TRANSACTION_PAYMENT_GATEWAY_ID: "Payment gateway ID of transaction",
            cls.TRANSACTION_PAYMENT_GATEWAY_STATUS: "Payment gateway status of transaction",
            cls.TRANSACTION_PAYMENT_GATEWAY_MESSAGE: "Payment gateway message of transaction",
            cls.TRANSACTION_PAYMENT_GATEWAY_ERROR: "Payment gateway error of transaction",
            cls.TRANSACTION_PAYMENT_GATEWAY_ERROR_MESSAGE: "Payment gateway error message of transaction",
            cls.TRANSACTION_PAYMENT_GATEWAY_ERROR_CODE: "Payment gateway error code of transaction",
            cls.TRANSACTION_PAYMENT_GATEWAY_ERROR_DETAILS: "Payment gateway error details of transaction"
        }
    
    
class ReferralEmailVariables:
    """Variables specific to referral-related emails"""
    REFERRAL_TYPE = "referral_type"
    REFERRAL_STATUS = "referral_status"
    REFERRAL_CODE = "referral_code"
    REFERRAL_EVENT = "referral_event"
    REFERRAL_MESSAGE = "referral_message"
    REFERRAL_ID = "referral_id"
    REFERRAL_AMOUNT = "referral_amount"
    REFERRAL_CAMPAIGN = "referral_campaign"
    REFERRAL_SOURCE = "referral_source"
    REFERRAL_PAYMENT_METHOD = "referral_payment_method"
    REFERRAL_PAYMENT_GATEWAY = "referral_payment_gateway"
    REFERRAL_CUSTOMER_NAME = "referral_customer_name"
    REFERRAL_CUSTOMER_EMAIL = "referral_customer_email"
    REFERRAL_CUSTOMER_PHONE = "referral_customer_phone"
    REFERRAL_CUSTOMER_COUNTRY = "referral_customer_country"

    REFERRAL_LINK = "referral_link"
    REFERRAL_CURRENCY = "referral_currency"

    @classmethod
    def get_all_variables(cls) -> list:
        return [
            getattr(cls, attr) for attr in dir(cls)
            if not attr.startswith('_') and isinstance(getattr(cls, attr), str)
        ]
    
    @classmethod
    def get_variable_descriptions(cls) -> dict:
        return {
            cls.REFERRAL_TYPE: "Type of referral",
            cls.REFERRAL_STATUS: "Status of referral",
            cls.REFERRAL_CODE: "Code of referral",
            cls.REFERRAL_EVENT: "Event of referral",
            cls.REFERRAL_MESSAGE: "Message of referral",
            cls.REFERRAL_ID: "ID of referral",
            cls.REFERRAL_AMOUNT: "Amount of referral",
            cls.REFERRAL_CAMPAIGN: "Campaign of referral",
            cls.REFERRAL_SOURCE: "Source of referral",
            cls.REFERRAL_PAYMENT_METHOD: "Payment method of referral",
            cls.REFERRAL_PAYMENT_GATEWAY: "Payment gateway of referral",
            cls.REFERRAL_CUSTOMER_NAME: "Name of referral customer",
            cls.REFERRAL_CUSTOMER_EMAIL: "Email of referral customer",
            cls.REFERRAL_CUSTOMER_PHONE: "Phone of referral customer",
            cls.REFERRAL_CUSTOMER_COUNTRY: "Country of referral customer",
            cls.REFERRAL_LINK: "The unique referral link to share",
            cls.REFERRAL_CURRENCY: "The currency of the referral reward amount",
        }
        

class TicketEmailVariables:
    """Variables specific to ticket-related emails"""
    TICKET_TYPE = "ticket_type"
    TICKET_STATUS = "ticket_status"
    TICKET_MESSAGE = "ticket_message"
    TICKET_ID = "ticket_id"
    TICKET_SUBJECT = "ticket_subject"
    TICKET_CATEGORY = "ticket_category"
    TICKET_PRIORITY = "ticket_priority"
    TICKET_ASSIGNEE = "ticket_assignee"
    TICKET_MESSAGE = "ticket_message"
    TICKET_CREATED_AT = "ticket_created_at"
    TICKET_UPDATED_AT = "ticket_updated_at"
    TICKET_CLOSED_AT = "ticket_closed_at"
    TICKET_RESOLVED_AT = "ticket_resolved_at"
    TICKET_RESOLVED_BY = "ticket_resolved_by"
    TICKET_RESOLVED_MESSAGE = "ticket_resolved_message"
    

    @classmethod
    def get_all_variables(cls) -> list:
        return [
            getattr(cls, attr) for attr in dir(cls)
            if not attr.startswith('_') and isinstance(getattr(cls, attr), str)
        ]
    
    @classmethod
    def get_variable_descriptions(cls) -> dict:
        return {
            cls.TICKET_TYPE: "Type of ticket",
            cls.TICKET_STATUS: "Status of ticket",
            cls.TICKET_MESSAGE: "Message of ticket",
            cls.TICKET_ID: "ID of ticket",
            cls.TICKET_SUBJECT: "Subject of ticket",
            cls.TICKET_CATEGORY: "Category of ticket",
            cls.TICKET_PRIORITY: "Priority of ticket",
            cls.TICKET_ASSIGNEE: "Assignee of ticket",
            cls.TICKET_CREATED_AT: "Created at of ticket",
            cls.TICKET_UPDATED_AT: "Updated at of ticket",
            cls.TICKET_CLOSED_AT: "Closed at of ticket",
            cls.TICKET_RESOLVED_AT: "Resolved at of ticket",
            cls.TICKET_RESOLVED_BY: "Resolved by of ticket",
            cls.TICKET_RESOLVED_MESSAGE: "Resolved message of ticket"
        }
    
class RequestEmailVariables:
    """Variables specific to request-related emails"""
    REQUEST_TYPE = "request_type"
    REQUEST_STATUS = "request_status"
    REQUEST_MESSAGE = "request_message"
    REQUEST_ID = "request_id"

    @classmethod
    def get_all_variables(cls) -> list:
        return [
            getattr(cls, attr) for attr in dir(cls)
            if not attr.startswith('_') and isinstance(getattr(cls, attr), str)
        ]
    
    @classmethod
    def get_variable_descriptions(cls) -> dict:
        return {
            cls.REQUEST_TYPE: "Type of request",
            cls.REQUEST_STATUS: "Status of request",
            cls.REQUEST_MESSAGE: "Message of request",
            cls.REQUEST_ID: "ID of request"
        }

class SystemEmailVariables:
    """Variables specific to system-related emails"""
    ALERT_TYPE = "alert_type"
    ALERT_MESSAGE = "alert_message"
    SEVERITY = "severity"
    ACTION_REQUIRED = "action_required"
    MAINTENANCE_START = "maintenance_start"
    MAINTENANCE_END = "maintenance_end"
    AFFECTED_SERVICES = "affected_services"
    SYSTEM_STATUS = "system_status"
    ERROR_DETAILS = "error_details"
    RESOLUTION_STEPS = "resolution_steps"

    @classmethod
    def get_all_variables(cls) -> list:
        return [
            getattr(cls, attr) for attr in dir(cls)
            if not attr.startswith('_') and isinstance(getattr(cls, attr), str)
        ]

    @classmethod
    def get_variable_descriptions(cls) -> dict:
        return {
            cls.ALERT_TYPE: "Type of system alert",
            cls.ALERT_MESSAGE: "Alert message details",
            cls.SEVERITY: "Severity level of the alert",
            cls.ACTION_REQUIRED: "Required action to be taken",
            cls.MAINTENANCE_START: "Start time of maintenance",
            cls.MAINTENANCE_END: "End time of maintenance",
            cls.AFFECTED_SERVICES: "Services affected by maintenance/issue",
            cls.SYSTEM_STATUS: "Current system status",
            cls.ERROR_DETAILS: "Detailed error information",
            cls.RESOLUTION_STEPS: "Steps to resolve the issue"
        }

class TriggerVariableMapping:
    """Maps trigger events to their specific variable classes"""
    MAPPINGS = {
        # Authentication triggers
        "login_notification": AuthenticationEmailVariables,
        "login_failed_notification": AuthenticationEmailVariables,
        "password_reset": AuthenticationEmailVariables,
        "password_reset_request": AuthenticationEmailVariables,
        "email_verification": AuthenticationEmailVariables,
        "two_factor_code": AuthenticationEmailVariables,
        "access_changed": AuthenticationEmailVariables,
        "two_factor_status_changed": AuthenticationEmailVariables,
        "security_key_status_changed": AuthenticationEmailVariables,
        
        # Customer triggers
        "customer_welcome_live_email": CustomerEmailVariables,
        "customer_welcome_demo_email": CustomerEmailVariables,
        "customer_welcome_ib_email": CustomerEmailVariables,
        "customer_agent_assigned": CustomerEmailVariables,
        "customer_agent_unassigned": CustomerEmailVariables,
        "customer_feedback": CustomerEmailVariables,
        "customer_support": CustomerEmailVariables,

        "account_updated": AccountEmailVariables,
        "account_created": AccountEmailVariables,
        "account_linked": AccountEmailVariables,
        "account_status_changed": AccountEmailVariables,
        "account_trading_status_changed": AccountEmailVariables,
        "account_password_changed": AccountEmailVariables,
        "account_reset_password": AccountEmailVariables,
        "account_leverage_changed": AccountEmailVariables,
        "account_group_changed": AccountEmailVariables,
        "account_archived": AccountEmailVariables,
        "account_restored": AccountEmailVariables,

        "document_uploaded": DocumentEmailVariables,
        "document_status_changed": DocumentEmailVariables,
        "document_verification_required": DocumentEmailVariables,

        "kyc_submitted": KYCEmailVariables,
        "kyc_approved": KYCEmailVariables,
        "kyc_rejected": KYCEmailVariables,
        "kyc_resubmitted": KYCEmailVariables,

        "transaction_created": TransactionEmailVariables,
        "transaction_status_changed": TransactionEmailVariables,
        "transaction_failed": TransactionEmailVariables,
        "transaction_success": TransactionEmailVariables,
        "transaction_rejected": TransactionEmailVariables,
        "transaction_evidence_required": TransactionEmailVariables,
        "transaction_evidence_uploaded": TransactionEmailVariables,
        "transaction_evidence_verified": TransactionEmailVariables,
        "transaction_evidence_rejected": TransactionEmailVariables,
        
        "referral_created": ReferralEmailVariables,
        "referral_acquisition": ReferralEmailVariables,
        "referral_event": ReferralEmailVariables,
        "referral_reward_earned": ReferralEmailVariables,
        "referral_reward_payment_failed": ReferralEmailVariables,
        "referral_reward_payment_success": ReferralEmailVariables,

        "ticket_created": TicketEmailVariables,
        "ticket_status_changed": TicketEmailVariables,
        "ticket_priority_changed": TicketEmailVariables,
        "ticket_assigned": TicketEmailVariables,
        "ticket_new_message": TicketEmailVariables,
        "ticket_closed": TicketEmailVariables,
        
        # System triggers
        "system_alert": SystemEmailVariables,
        "maintenance_notification": SystemEmailVariables,
        "security_alert": SystemEmailVariables
    }

    @classmethod
    def get_variables_for_trigger(cls, trigger_event: str) -> tuple:
        """Get all available variables for a trigger event"""
        # Always include base variables
        variables = BaseEmailVariables.get_all_variables()
        descriptions = BaseEmailVariables.get_variable_descriptions()
        
        # Add trigger-specific variables
        if trigger_event in cls.MAPPINGS:
            variable_class = cls.MAPPINGS[trigger_event]
            variables.extend(variable_class.get_all_variables())
            descriptions.update(variable_class.get_variable_descriptions())
        
        return variables, descriptions 
    
class AccountType(str, Enum):
    LIVE = 'LIVE'
    DEMO = 'DEMO'
    IB = 'IB'

    @classmethod
    def get_all_types(cls) -> list:
        return [
            getattr(cls, attr) for attr in dir(cls)
            if not attr.startswith('_') and isinstance(getattr(cls, attr), str)
        ]

# --- Flattened AllSystemVariables (Shared Models Version) ---
class AllSystemVariables:
    """Flat access to all system email variables defined in shared_models, prioritizing specific over base."""
    _all_vars = {}

    # Order matters: Start with base, then overwrite with specifics
    _variable_classes = [
        BaseEmailVariables, # Uses BaseEmailVariables from this file
        AuthenticationEmailVariables,
        CustomerEmailVariables,
        SystemEmailVariables,
        AccountEmailVariables,
        DocumentEmailVariables,
        KYCEmailVariables,
        TransactionEmailVariables,
        ReferralEmailVariables, 
        TicketEmailVariables,
        RequestEmailVariables
    ]

    for var_class in _variable_classes:
        # Check if the class itself exists before iterating
        if var_class:
            for attr in dir(var_class):
                if not attr.startswith('_') and isinstance(getattr(var_class, attr, None), str):
                    _all_vars[attr] = getattr(var_class, attr)

    # Dynamically set class attributes from the aggregated dictionary
    for key, value in _all_vars.items():
        locals()[key] = value
        
    # Clean up temporary variables used during class creation
    del _all_vars
    del _variable_classes
    del var_class
    del attr
    del key
    del value




# Generated by Django 5.1.4 on 2025-01-17 13:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('system_settings', '0003_tradingplatformserver'),
    ]

    operations = [
        migrations.AddField(
            model_name='tradingplatformserver',
            name='connection_timeout',
            field=models.IntegerField(default=30),
        ),
        migrations.AddField(
            model_name='tradingplatformserver',
            name='health_check_interval',
            field=models.IntegerField(default=30),
        ),
        migrations.AddField(
            model_name='tradingplatformserver',
            name='is_connected',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='tradingplatformserver',
            name='max_retries',
            field=models.IntegerField(default=3),
        ),
        migrations.AddField(
            model_name='tradingplatformserver',
            name='retry_max_wait',
            field=models.IntegerField(default=30),
        ),
        migrations.AddField(
            model_name='tradingplatformserver',
            name='retry_min_wait',
            field=models.IntegerField(default=1),
        ),
    ]



# Generated by Django 5.1.4 on 2025-02-08 12:42

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('accounts', '0003_account_is_archived_account_is_trading_enabled'),
        ('customers', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Transaction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('amount', models.DecimalField(decimal_places=2, max_digits=20)),
                ('currency', models.CharField(max_length=3)),
                ('transaction_type', models.CharField(choices=[('DEPOSIT', 'Deposit'), ('WITHDRAWAL', 'Withdrawal'), ('BONUS', 'Bonus'), ('REBATE', 'Rebate'), ('COMMISSION', 'Commission'), ('CORRECTION', 'Correction'), ('INTERNAL_TRANSFER', 'Internal Transfer'), ('CREDIT', 'Credit'), ('CHARGE', 'Charge'), ('COMPENSATION', 'Compensation')], max_length=20)),
                ('direction', models.CharField(choices=[('IN', 'Credit/Incoming'), ('OUT', 'Debit/Outgoing')], max_length=3)),
                ('status', models.CharField(choices=[('PENDING', 'Pending'), ('APPROVED', 'Approved'), ('REJECTED', 'Rejected'), ('CANCELLED', 'Cancelled'), ('FAILED', 'Failed')], default='PENDING', max_length=10)),
                ('payment_method', models.CharField(blank=True, choices=[('WIRE', 'Wire Transfer'), ('CRYPTO', 'Cryptocurrency'), ('CARD', 'Credit/Debit Card'), ('MANUAL', 'Manual'), ('SYSTEM', 'System Generated')], max_length=10, null=True)),
                ('payment_gateway', models.CharField(max_length=50)),
                ('gateway_fees', models.DecimalField(blank=True, decimal_places=2, max_digits=10, null=True)),
                ('payment_reference', models.CharField(blank=True, max_length=255, null=True)),
                ('notes', models.TextField(blank=True)),
                ('rejection_reason', models.TextField(blank=True)),
                ('metadata', models.JSONField(blank=True, default=dict)),
                ('account', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='transactions', to='accounts.account')),
                ('approved_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='approved_transactions', to=settings.AUTH_USER_MODEL)),
                ('created_by', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='created_transactions', to=settings.AUTH_USER_MODEL)),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='transactions', to='customers.customer')),
            ],
            options={
                'db_table': 'transactions',
                'ordering': ['-created_at'],
            },
        ),
        migrations.CreateModel(
            name='TransactionEvidence',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('file', models.FileField(upload_to='transaction_evidences/%Y/%m/%d/')),
                ('verified', models.BooleanField(default=False)),
                ('verified_at', models.DateTimeField(blank=True, null=True)),
                ('metadata', models.JSONField(blank=True, default=dict)),
                ('transaction', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='evidences', to='transactions.transaction')),
            ],
            options={
                'db_table': 'transaction_evidences',
                'ordering': ['-created_at'],
            },
        ),
        migrations.CreateModel(
            name='CommissionRebateTransaction',
            fields=[
                ('transaction_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='transactions.transaction')),
                ('calculation_basis', models.JSONField(help_text='Parameters used for calculation')),
                ('ib_account', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='commission_rebate_transactions', to='accounts.account')),
            ],
            options={
                'db_table': 'commission_rebate_transactions',
            },
            bases=('transactions.transaction',),
        ),
        migrations.AddIndex(
            model_name='transaction',
            index=models.Index(fields=['transaction_type', 'status'], name='transaction_transac_9f5934_idx'),
        ),
        migrations.AddIndex(
            model_name='transaction',
            index=models.Index(fields=['account', 'created_at'], name='transaction_account_6c549c_idx'),
        ),
        migrations.AddIndex(
            model_name='transaction',
            index=models.Index(fields=['payment_method'], name='transaction_payment_afdbde_idx'),
        ),
        migrations.AddIndex(
            model_name='transaction',
            index=models.Index(fields=['payment_gateway'], name='transaction_payment_050505_idx'),
        ),
    ]



from django.db import models
from shared_models.accounts.models import Account
from shared_models.common.base import BaseModel

class Deal(BaseModel):
    deal_ticket = models.BigIntegerField(unique=True, db_index=True)
    external_id = models.CharField(max_length=100, null=True, blank=True, db_index=True)
    login = models.BigIntegerField(db_index=True)
    dealer = models.BigIntegerField(db_index=True)
    order_ticket = models.BigIntegerField(db_index=True)
    action = models.IntegerField()
    entry = models.IntegerField(db_index=True)
    digits = models.IntegerField()
    digits_currency = models.IntegerField()
    contract_size = models.DecimalField(max_digits=20, decimal_places=10)
    time = models.DateTimeField(db_index=True)
    time_msc = models.BigIntegerField(db_index=True)
    symbol = models.CharField(max_length=50, db_index=True)
    price = models.DecimalField(max_digits=20, decimal_places=10)
    price_sl = models.DecimalField(max_digits=20, decimal_places=10)
    price_tp = models.DecimalField(max_digits=20, decimal_places=10)
    volume = models.DecimalField(max_digits=20, decimal_places=10)
    volume_ext = models.DecimalField(max_digits=30, decimal_places=15)
    volume_closed = models.DecimalField(max_digits=20, decimal_places=10)
    volume_closed_ext = models.DecimalField(max_digits=30, decimal_places=15)
    profit = models.DecimalField(max_digits=20, decimal_places=10)
    profit_raw = models.DecimalField(max_digits=20, decimal_places=10)
    storage = models.DecimalField(max_digits=20, decimal_places=10)
    commission = models.DecimalField(max_digits=20, decimal_places=10)
    fee = models.DecimalField(max_digits=20, decimal_places=10)
    value = models.DecimalField(max_digits=20, decimal_places=10)
    rate_profit = models.DecimalField(max_digits=20, decimal_places=10)
    rate_margin = models.DecimalField(max_digits=20, decimal_places=10)
    expert_id = models.BigIntegerField(null=True, blank=True)
    position_id = models.BigIntegerField(db_index=True)
    comment = models.CharField(max_length=255, null=True, blank=True)
    price_position = models.DecimalField(max_digits=20, decimal_places=10)
    tick_value = models.DecimalField(max_digits=20, decimal_places=10)
    tick_size = models.DecimalField(max_digits=20, decimal_places=10)
    flags = models.BigIntegerField()
    reason = models.IntegerField(null=True, blank=True, db_index=True)
    gateway = models.CharField(max_length=100, null=True, blank=True)
    price_gateway = models.DecimalField(max_digits=20, decimal_places=10, null=True, blank=True)
    market_bid = models.DecimalField(max_digits=20, decimal_places=10)
    market_ask = models.DecimalField(max_digits=20, decimal_places=10)
    market_last = models.DecimalField(max_digits=20, decimal_places=10)
    modification_flags = models.IntegerField()
    raw_data = models.JSONField(default=dict, blank=True)
    account = models.ForeignKey(Account, on_delete=models.SET_NULL, null=True, blank=True, db_index=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        indexes = [
            models.Index(fields=["deal_ticket"]),
            models.Index(fields=["login"]),
            models.Index(fields=["account"]),
            models.Index(fields=["time"]),
            models.Index(fields=["time_msc"]),
            models.Index(fields=["position_id"]),
            models.Index(fields=["symbol"]),
            models.Index(fields=["entry"]),
            models.Index(fields=["reason"]),
            models.Index(fields=['position_id', 'login', 'entry'], name='idx_deal_position_lookup'),
            models.Index(fields=['login', 'time'], name='idx_deal_login_time'),
        ]
        db_table = "deal"
        verbose_name = "Deal"
        verbose_name_plural = "Deals" 


# Generated by Django 4.2.21 on 2025-06-26 13:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('trading', '0004_alter_deal_id'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='deal',
            index=models.Index(fields=['position_id', 'login', 'entry'], name='idx_deal_position_lookup'),
        ),
        migrations.AddIndex(
            model_name='deal',
            index=models.Index(fields=['login', 'time'], name='idx_deal_login_time'),
        ),
    ]



# Generated by Django 4.2.20 on 2025-04-22 15:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('system_settings', '0014_alter_paymentgatewayconfig_payment_method_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailCampaign',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('id', models.AutoField(editable=False, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255, verbose_name='Campaign Name')),
                ('schedule_type', models.CharField(max_length=100, verbose_name='Schedule Type')),
                ('schedule_time', models.DateTimeField(verbose_name='Schedule Time')),
                ('status', models.CharField(choices=[('DRAFT', 'Draft'), ('SCHEDULED', 'Scheduled'), ('IN_PROGRESS', 'In Progress'), ('COMPLETED', 'Completed'), ('FAILED', 'Failed'), ('CANCELLED', 'Cancelled')], default='DRAFT', max_length=20, verbose_name='Status')),
                ('available_variables', models.JSONField(default=list, help_text='Variables that can be used in email content (e.g., ["customer_name", "account_number"])', verbose_name='Available Variables')),
                ('filters', models.JSONField(blank=True, default=dict, help_text='Flexible filtering criteria for targeting email recipients. Can include country, kycStatus, leadStatus, callStatus, isFunded, customerType, source, agentId, ibId, createdAtFrom, createdAtTo, and any other future filter criteria.', verbose_name='Filters')),
                ('template', models.ForeignKey(help_text='Email template with placeholders for content sections', on_delete=django.db.models.deletion.PROTECT, to='system_settings.emailtemplate')),
            ],
            options={
                'verbose_name': 'Email Campaign',
                'verbose_name_plural': 'Email Campaigns',
                'db_table': 'shared_email_campaign',
                'ordering': ['-created_at'],
                'permissions': [('view_email_campaign', 'Can view email campaign'), ('delete_email_campaign', 'Can delete email campaign'), ('change_email_campaign', 'Can change email campaign'), ('add_email_campaign', 'Can add email campaign')],
            },
        ),
        migrations.CreateModel(
            name='EmailCampaignEvent',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('id', models.AutoField(editable=False, primary_key=True, serialize=False)),
                ('event_type', models.CharField(choices=[('SCHEDULED', 'Scheduled'), ('EDITED', 'Edited'), ('STARTING', 'Starting'), ('WARNING', 'Warning'), ('FAILED', 'Failed'), ('COMPLETED', 'Completed')], max_length=20, verbose_name='Event Type')),
                ('details', models.JSONField(blank=True, default=dict, help_text='Additional details about the event, such as error messages, success counts, etc.', verbose_name='Event Details')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
                ('campaign', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='events', to='shared_email_campaign.emailcampaign')),
            ],
            options={
                'verbose_name': 'Email Campaign Event',
                'verbose_name_plural': 'Email Campaign Events',
                'db_table': 'shared_email_campaign_event',
                'ordering': ['-created_at'],
                'indexes': [models.Index(fields=['campaign'], name='shared_emai_campaig_6ca583_idx'), models.Index(fields=['event_type'], name='shared_emai_event_t_358605_idx'), models.Index(fields=['created_at'], name='shared_emai_created_6ec312_idx')],
            },
        ),
        migrations.CreateModel(
            name='EmailCampaignContent',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('id', models.AutoField(editable=False, primary_key=True, serialize=False)),
                ('language_code', models.CharField(help_text='e.g., "en", "es", "fr"', max_length=10, verbose_name='Language Code')),
                ('subject', models.CharField(max_length=255, verbose_name='Email Subject')),
                ('content_sections', models.JSONField(default=dict, help_text='Dict mapping template placeholders to content structures. Each structure has "default" content and optional "condition_sets". Content can include {{ variables }}', verbose_name='Content Sections')),
                ('content_type', models.CharField(choices=[('HTML', 'HTML'), ('TEXT', 'TEXT')], default='HTML', max_length=20, verbose_name='Content Type')),
                ('campaign', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='contents', to='shared_email_campaign.emailcampaign')),
            ],
            options={
                'verbose_name': 'Email Campaign Content',
                'verbose_name_plural': 'Email Campaign Contents',
                'db_table': 'shared_email_campaign_content',
                'ordering': ['-created_at'],
                'permissions': [('view_email_campaign_content', 'Can view email campaign content'), ('delete_email_campaign_content', 'Can delete email campaign content'), ('change_email_campaign_content', 'Can change email campaign content'), ('add_email_campaign_content', 'Can add email campaign content')],
                'indexes': [models.Index(fields=['campaign'], name='shared_emai_campaig_442532_idx'), models.Index(fields=['language_code'], name='shared_emai_languag_5d3689_idx')],
                'unique_together': {('campaign', 'language_code', 'content_type')},
            },
        ),
        migrations.AddIndex(
            model_name='emailcampaign',
            index=models.Index(fields=['name'], name='shared_emai_name_95ed16_idx'),
        ),
        migrations.AddIndex(
            model_name='emailcampaign',
            index=models.Index(fields=['schedule_type'], name='shared_emai_schedul_88b233_idx'),
        ),
        migrations.AddIndex(
            model_name='emailcampaign',
            index=models.Index(fields=['schedule_time'], name='shared_emai_schedul_571a3c_idx'),
        ),
        migrations.AddIndex(
            model_name='emailcampaign',
            index=models.Index(fields=['status'], name='shared_emai_status_bfa128_idx'),
        ),
    ]



# Generated by Django 4.2.19 on 2025-02-19 07:43

from django.db import migrations
import encrypted_model_fields.fields


class Migration(migrations.Migration):

    dependencies = [
        ('customers', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customer',
            name='email',
            field=encrypted_model_fields.fields.EncryptedEmailField(unique=True, verbose_name='email address'),
        ),
        migrations.AlterField(
            model_name='customer',
            name='first_name',
            field=encrypted_model_fields.fields.EncryptedCharField(blank=True),
        ),
        migrations.AlterField(
            model_name='customer',
            name='last_name',
            field=encrypted_model_fields.fields.EncryptedCharField(blank=True),
        ),
        migrations.AlterField(
            model_name='customer',
            name='mobile',
            field=encrypted_model_fields.fields.EncryptedCharField(blank=True),
        ),
        migrations.AlterField(
            model_name='customer',
            name='phone',
            field=encrypted_model_fields.fields.EncryptedCharField(blank=True),
        ),
        migrations.AlterField(
            model_name='customerdetail',
            name='address',
            field=encrypted_model_fields.fields.EncryptedCharField(blank=True),
        ),
        migrations.AlterField(
            model_name='customerdetail',
            name='address2',
            field=encrypted_model_fields.fields.EncryptedCharField(blank=True),
        ),
        migrations.AlterField(
            model_name='customerdetail',
            name='city',
            field=encrypted_model_fields.fields.EncryptedCharField(blank=True),
        ),
        migrations.AlterField(
            model_name='customerdetail',
            name='title',
            field=encrypted_model_fields.fields.EncryptedCharField(blank=True),
        ),
        migrations.AlterField(
            model_name='customerdetail',
            name='zip_code',
            field=encrypted_model_fields.fields.EncryptedCharField(blank=True),
        ),
    ]



from .base import SystemSetting
from .email import EmailConfiguration, EmailTemplate, SystemEmail, EmailContent
from .trading_platform import TradingPlatformServer, TradingPlatformServerLog
from .account_types import AccountType
from .account_sequence import AccountSequence
from .group_configurations import GroupConfiguration, IBCustomGroupConfiguration
from .transaction_configs import TransactionTypeConfig
from .payment_methods import PaymentGatewayConfig, PaymentMethodConfig, KycRequirementRule
from .broker_config import BrokerConfiguration


__all__ = [
    'SystemSetting',
    'EmailConfiguration',
    'EmailTemplate',
    'SystemEmail',
    'EmailContent',
    'TradingPlatformServer',
    'AccountType',
    'AccountSequence',
    'GroupConfiguration',
    'IBCustomGroupConfiguration',
    'TransactionTypeConfig',
    'PaymentGatewayConfig',
    'PaymentMethodConfig',
    'KycRequirementRule',
    'TradingPlatformServerLog',
    'BrokerConfiguration',
]



import logging

logger = logging.getLogger('commission_cache')

def get_cache():
    """
    Lazy import of Django cache to ensure it uses the consuming application's cache configuration.
    This prevents using the dummy cache from library_settings.py.
    """
    from django.core.cache import cache
    return cache

class CommissionCacheService:
    """Service for caching commission-related data using Django's cache framework"""
    
    # Cache key patterns (use consistent patterns across projects)
    RULE_CACHE_KEY = "commission_rule:{agreement_id}:{symbol}:{commission_type}"
    AGREEMENT_CACHE_KEY = "ib_agreement:{customer_id}"
    HIERARCHY_CACHE_KEY = "ib_hierarchy:{customer_id}"
    CLIENT_MAPPING_CACHE_KEY = "client_mapping:{mt5_login}:{customer_id}"
    ACCOUNT_CACHE_KEY = "account:{mt5_login}"
    
    # Cache expiration times (in seconds)
    RULE_CACHE_EXPIRY = 3600 * 24  # 24 hours
    AGREEMENT_CACHE_EXPIRY = 3600 * 24  # 24 hours
    HIERARCHY_CACHE_EXPIRY = 3600 * 24  # 24 hours
    CLIENT_MAPPING_CACHE_EXPIRY = 3600 * 24  # 24 hours
    ACCOUNT_CACHE_EXPIRY = 3600 * 24  # 24 hours
    
    # Flag to enable/disable cache
    CACHE_ENABLED = True
    
    @classmethod
    def get_account(cls, mt5_login):
        """
        Get account from cache or database
        
        Args:
            mt5_login: The MT5 login
            
        Returns:
            Account object or None
        """
        if not cls.CACHE_ENABLED:
            logger.info("Cache disabled, fetching account from database")
            return cls._get_account_from_db(mt5_login)
            
        from shared_models.accounts.models import Account
        
        cache_key = cls.ACCOUNT_CACHE_KEY.format(mt5_login=mt5_login)
        
        logger.info(f"Looking for account cache key: {cache_key}")
        
        # Try to get from cache first
        account = get_cache().get(cache_key)
        if account:
            logger.info(f"Account cache hit for {cache_key}")
            return account
        
        logger.info(f"Account cache miss for {cache_key}")
        
        # If not in cache, get from database
        account = cls._get_account_from_db(mt5_login)
        
        if account:
            # Store in cache for future use
            logger.info(f"Storing account in cache: {cache_key}")
            get_cache().set(cache_key, account, cls.ACCOUNT_CACHE_EXPIRY)
        
        return account
    
    @classmethod
    def _get_account_from_db(cls, mt5_login):
        """Get account directly from database"""
        from shared_models.accounts.models import Account
        
        logger.info(f"Fetching account from DB for mt5_login={mt5_login}")
        
        account = Account.objects.filter(login=mt5_login, is_active=True).first()
        
        logger.info(f"Found account in database: {account is not None}")
        return account
    
    @classmethod
    def invalidate_account_cache(cls, mt5_login):
        """
        Invalidate account cache
        
        Args:
            mt5_login: The MT5 login
        """
        if not cls.CACHE_ENABLED:
            logger.info("Cache disabled, skipping account invalidation")
            return
            
        cache_key = cls.ACCOUNT_CACHE_KEY.format(mt5_login=mt5_login)
        logger.info(f"Invalidating account cache key: {cache_key}")
        get_cache().delete(cache_key)
    
    @classmethod
    def get_commission_rules(cls, agreement_id, symbol, commission_type, account_type_id=None):
        """
        Get commission rules from cache or database
        
        Args:
            agreement_id: The agreement ID
            symbol: The trading symbol
            commission_type: The commission type (COMMISSION or REBATE)
            account_type_id: Optional account type ID for more specific caching
            
        Returns:
            List of commission rules
        """
        if not cls.CACHE_ENABLED:
            logger.info("Cache disabled, fetching from database")
            return cls._get_commission_rules_from_db(agreement_id, symbol, commission_type, account_type_id)
            
        from shared_models.ib_commission.models import IBCommissionRule
        from django.db import models
        
        cache_key = cls.RULE_CACHE_KEY.format(
            agreement_id=agreement_id,
            symbol=symbol if symbol is not None else "none",
            commission_type=commission_type
        )
        
        # Add account type to cache key if provided
        if account_type_id:
            cache_key += f":at{account_type_id}"
        
        logger.info(f"Looking for cache key: {cache_key}")
        
        # Try to get from cache first
        rules = get_cache().get(cache_key)
        if rules is not None:
            logger.info(f"Cache hit for {cache_key}")
            return rules
        
        logger.info(f"Cache miss for {cache_key}")
        
        # If not in cache, get from database
        rules = cls._get_commission_rules_from_db(agreement_id, symbol, commission_type, account_type_id)
        
        # Store in cache for future use
        logger.info(f"Storing in cache: {cache_key}")
        get_cache().set(cache_key, rules, cls.RULE_CACHE_EXPIRY)
        
        return rules
    
    @classmethod
    def _get_commission_rules_from_db(cls, agreement_id, symbol, commission_type, account_type_id=None):
        """Get commission rules directly from database with better query optimization"""
        from shared_models.ib_commission.models import IBCommissionRule
        from django.db import models
        
        logger.info(f"Fetching rules from DB for agreement_id={agreement_id}, symbol={symbol}, commission_type={commission_type}, account_type_id={account_type_id}")
        
        rules_query = IBCommissionRule.objects.filter(
            agreement_id=agreement_id,
            commission_type=commission_type
        )
        
        # Smart filtering based on symbol
        if symbol and symbol != '*':
            rules_query = rules_query.filter(
                models.Q(symbol__iexact=symbol) | models.Q(symbol='*') | models.Q(symbol__isnull=True)
            )
        
        # Filter by account type if provided
        if account_type_id:
            rules_query = rules_query.filter(
                models.Q(account_type_id=account_type_id) | models.Q(account_type__isnull=True)
            )
        
        rules = list(rules_query.select_related('account_type').order_by('priority'))
        
        logger.info(f"Found {len(rules)} rules in database")
        return rules
    
    @classmethod
    def get_ib_agreements(cls, customer_id):
        """
        Get IB agreements from cache or database
        
        Args:
            customer_id: The customer ID
            
        Returns:
            List of agreement memberships
        """
        if not cls.CACHE_ENABLED:
            logger.info("Cache disabled, fetching from database")
            return cls._get_ib_agreements_from_db(customer_id)
            
        from shared_models.ib_commission.models import IBAgreementMember
        
        cache_key = cls.AGREEMENT_CACHE_KEY.format(customer_id=customer_id)
        
        logger.info(f"Looking for cache key: {cache_key}")
        
        # Try to get from cache first
        agreements = get_cache().get(cache_key)
        if agreements:
            logger.info(f"Cache hit for {cache_key}")
            return agreements
        
        logger.info(f"Cache miss for {cache_key}")
        
        # If not in cache, get from database
        agreements = cls._get_ib_agreements_from_db(customer_id)
        
        # Store in cache for future use
        logger.info(f"Storing in cache: {cache_key}")
        get_cache().set(cache_key, agreements, cls.AGREEMENT_CACHE_EXPIRY)
        
        return agreements
    
    @classmethod
    def _get_ib_agreements_from_db(cls, customer_id):
        """Get IB agreements directly from database"""
        from shared_models.ib_commission.models import IBAgreementMember
        
        logger.info(f"Fetching agreements from DB for customer_id={customer_id}")
        
        agreements = list(IBAgreementMember.objects.filter(
            customer_id=customer_id,
            is_active=True
        ).select_related('agreement'))
        
        logger.info(f"Found {len(agreements)} agreements in database")
        return agreements
    
    @classmethod
    def get_ib_hierarchy(cls, customer_id):
        """
        Get IB hierarchy from cache or database
        
        Args:
            customer_id: The customer ID
            
        Returns:
            IBHierarchy object
        """
        if not cls.CACHE_ENABLED:
            logger.info("Cache disabled, fetching from database")
            return cls._get_ib_hierarchy_from_db(customer_id)
            
        from shared_models.ib_commission.models import IBHierarchy
        
        cache_key = cls.HIERARCHY_CACHE_KEY.format(customer_id=customer_id)
        
        logger.info(f"Looking for cache key: {cache_key}")
        
        # Try to get from cache first
        hierarchy = get_cache().get(cache_key)
        if hierarchy:
            logger.info(f"Cache hit for {cache_key}")
            return hierarchy
        
        logger.info(f"Cache miss for {cache_key}")
        
        # If not in cache, get from database
        hierarchy = cls._get_ib_hierarchy_from_db(customer_id)
        
        if hierarchy:
            # Store in cache for future use
            logger.info(f"Storing in cache: {cache_key}")
            get_cache().set(cache_key, hierarchy, cls.HIERARCHY_CACHE_EXPIRY)
        
        return hierarchy
    
    @classmethod
    def _get_ib_hierarchy_from_db(cls, customer_id):
        """Get IB hierarchy directly from database"""
        from shared_models.ib_commission.models import IBHierarchy
        
        logger.info(f"Fetching hierarchy from DB for customer_id={customer_id}")
        
        hierarchy = IBHierarchy.objects.filter(
            customer_id=customer_id,
            is_active=True
        ).first()
        
        logger.info(f"Found hierarchy in database: {hierarchy is not None}")
        return hierarchy
    
    @classmethod
    def get_client_mapping(cls, mt5_login, customer_id):
        """
        Get client mapping from cache or database
        
        Args:
            mt5_login: The MT5 login
            customer_id: The customer ID
            
        Returns:
            ClientIBMapping object
        """
        if not cls.CACHE_ENABLED:
            logger.info("Cache disabled, fetching from database")
            return cls._get_client_mapping_from_db(mt5_login, customer_id)
            
        from shared_models.ib_commission.models import ClientIBMapping
        
        cache_key = cls.CLIENT_MAPPING_CACHE_KEY.format(
            mt5_login=mt5_login or "none",
            customer_id=customer_id
        )
        
        logger.info(f"Looking for cache key: {cache_key}")
        
        # Try to get from cache first
        mapping = get_cache().get(cache_key)
        if mapping:
            logger.info(f"Cache hit for {cache_key}")
            return mapping
        
        logger.info(f"Cache miss for {cache_key}")
        
        # If not in cache, get from database
        mapping = cls._get_client_mapping_from_db(mt5_login, customer_id)
        
        if mapping:
            # Store in cache for future use
            logger.info(f"Storing in cache: {cache_key}")
            get_cache().set(cache_key, mapping, cls.CLIENT_MAPPING_CACHE_EXPIRY)
        
        return mapping
    
    @classmethod
    def _get_client_mapping_from_db(cls, mt5_login, customer_id):
        """Get client mapping directly from database"""
        from shared_models.ib_commission.models import ClientIBMapping
        
        logger.info(f"Fetching client mapping from DB for mt5_login={mt5_login}, customer_id={customer_id}")
        
        mapping = None
        if mt5_login:
            mapping = ClientIBMapping.objects.filter(
                mt5_login=mt5_login,
                customer_id=customer_id
            ).first()
        
        if not mapping and customer_id:
            # Try to find by customer only
            mapping = ClientIBMapping.objects.filter(
                customer_id=customer_id
            ).first()
        
        logger.info(f"Found client mapping in database: {mapping is not None}")
        return mapping
    
    @classmethod
    def invalidate_rule_cache(cls, agreement_id=None):
        """
        Invalidate rule cache for an agreement
        
        Args:
            agreement_id: The agreement ID (if None, invalidate all rule caches)
        """
        if not cls.CACHE_ENABLED:
            logger.info("Cache disabled, skipping invalidation")
            return
            
        if agreement_id:
            # Get all unique symbols and account types for this agreement from the database
            from shared_models.ib_commission.models import IBCommissionRule
            
            # Get all rules for this agreement
            rules = IBCommissionRule.objects.filter(
                agreement_id=agreement_id
            ).values('symbol', 'account_type_id').distinct()
            
            # Build list of unique symbol/account_type combinations
            symbol_account_combos = []
            unique_symbols = set()
            
            for rule in rules:
                symbol = rule['symbol']
                account_type_id = rule['account_type_id']
                unique_symbols.add(symbol)
                symbol_account_combos.append((symbol, account_type_id))
            
            # Also add None symbol if not present (for wildcard rules)
            if None not in unique_symbols:
                unique_symbols.add(None)
                # Add combinations for None symbol with all account types
                account_types = set(r['account_type_id'] for r in rules if r['account_type_id'])
                for at_id in account_types:
                    symbol_account_combos.append((None, at_id))
                # Also add None/None combo
                symbol_account_combos.append((None, None))
            
            # Invalidate all commission types, symbols, and account types for this agreement
            invalidated_count = 0
            for commission_type in ['COMMISSION', 'REBATE']:
                # First invalidate base keys (without account type)
                for symbol in unique_symbols:
                    cache_key = cls.RULE_CACHE_KEY.format(
                        agreement_id=agreement_id,
                        symbol=symbol if symbol is not None else "none",
                        commission_type=commission_type
                    )
                    logger.debug(f"Invalidating cache key: {cache_key}")
                    get_cache().delete(cache_key)
                    invalidated_count += 1
                
                # Then invalidate keys with account types
                for symbol, account_type_id in symbol_account_combos:
                    cache_key = cls.RULE_CACHE_KEY.format(
                        agreement_id=agreement_id,
                        symbol=symbol if symbol is not None else "none",
                        commission_type=commission_type
                    )
                    if account_type_id:
                        cache_key += f":at{account_type_id}"
                    logger.debug(f"Invalidating cache key: {cache_key}")
                    get_cache().delete(cache_key)
                    invalidated_count += 1
            
            logger.info(f"Invalidated {invalidated_count} cache keys for agreement {agreement_id}")
        else:
            # This is a more aggressive approach - clear the entire cache
            # Only use this if absolutely necessary
            logger.warning("Clearing entire cache")
            get_cache().clear()
    
    @classmethod
    def invalidate_agreement_cache(cls, customer_id):
        """
        Invalidate agreement cache for a customer
        
        Args:
            customer_id: The customer ID
        """
        if not cls.CACHE_ENABLED:
            logger.info("Cache disabled, skipping invalidation")
            return
            
        cache_key = cls.AGREEMENT_CACHE_KEY.format(customer_id=customer_id)
        logger.info(f"Invalidating cache key: {cache_key}")
        get_cache().delete(cache_key)
    
    @classmethod
    def invalidate_hierarchy_cache(cls, customer_id):
        """
        Invalidate hierarchy cache for a customer
        
        Args:
            customer_id: The customer ID
        """
        if not cls.CACHE_ENABLED:
            logger.info("Cache disabled, skipping invalidation")
            return
            
        cache_key = cls.HIERARCHY_CACHE_KEY.format(customer_id=customer_id)
        logger.info(f"Invalidating cache key: {cache_key}")
        get_cache().delete(cache_key)
    
    @classmethod
    def invalidate_client_mapping_cache(cls, mt5_login=None, customer_id=None):
        """
        Invalidate client mapping cache
        
        Args:
            mt5_login: The MT5 login
            customer_id: The customer ID
        """
        if not cls.CACHE_ENABLED:
            logger.info("Cache disabled, skipping invalidation")
            return
            
        if mt5_login and customer_id:
            cache_key = cls.CLIENT_MAPPING_CACHE_KEY.format(
                mt5_login=mt5_login,
                customer_id=customer_id
            )
            logger.info(f"Invalidating cache key: {cache_key}")
            get_cache().delete(cache_key)
        elif customer_id:
            # Get all mt5_logins for this customer from the database
            from shared_models.ib_commission.models import ClientIBMapping
            
            # Find all mt5_logins associated with this customer
            mt5_logins = ClientIBMapping.objects.filter(
                customer_id=customer_id
            ).values_list('mt5_login', flat=True).distinct()
            
            # Invalidate cache for each mt5_login
            invalidated_count = 0
            for mt5_login in mt5_logins:
                if mt5_login:  # Skip None values
                    cache_key = cls.CLIENT_MAPPING_CACHE_KEY.format(
                        mt5_login=mt5_login,
                        customer_id=customer_id
                    )
                    logger.info(f"Invalidating cache key: {cache_key}")
                    get_cache().delete(cache_key)
                    invalidated_count += 1
            
            logger.info(f"Invalidated {invalidated_count} client mapping cache keys for customer {customer_id}")
        else:
            logger.warning("Cannot invalidate client mapping cache without customer_id")
    
    @classmethod
    def list_all_cache_keys(cls):
        """
        List all cache keys (if possible)
        
        Returns:
            List of cache keys or None if not supported
        """
        try:
            # This only works with some cache backends like Redis
            cache = get_cache()
            if hasattr(cache, '_cache') and hasattr(cache._cache, 'keys'):
                try:
                    all_keys = cache._cache.keys('*')
                    logger.info(f"Found {len(all_keys)} keys in cache")
                    return all_keys
                except:
                    logger.warning("Could not retrieve keys from cache")
                    return None
            else:
                logger.warning("Cache backend does not support listing keys")
                return None
        except Exception as e:
            logger.exception(f"Error listing cache keys: {e}")
            return None 


# Generated by Django 4.2.19 on 2025-02-17 11:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0005_account_currency'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='account',
            options={'permissions': [('view_own_account', 'Can view own customer accounts'), ('view_team_account', "Can view team's customer accounts"), ('view_all_account', 'Can view all customer accounts'), ('change_account_group', 'Can change account group')], 'verbose_name': 'Account', 'verbose_name_plural': 'Accounts'},
        ),
    ]



# Generated by Django 5.1.4 on 2024-12-24 10:39

import django.contrib.auth.models
import django.contrib.auth.validators
import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='CRMUser',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('first_name', models.CharField(blank=True, max_length=150, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=150, verbose_name='last name')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('email', models.EmailField(max_length=254, unique=True, verbose_name='email address')),
                ('phone', models.CharField(blank=True, max_length=255, null=True)),
                ('mobile', models.CharField(blank=True, max_length=255, null=True)),
                ('is_blocked', models.BooleanField(default=False)),
                ('is_hidden', models.BooleanField(default=False)),
                ('is_deleted', models.BooleanField(default=False)),
                ('is_2fa_enabled', models.BooleanField(default=False)),
                ('two_factor_secret', models.CharField(blank=True, max_length=255, null=True)),
                ('temp_field', models.CharField(default='x', max_length=1)),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'db_table': 'crm_users',
                'abstract': False,
            },
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.AutoField(editable=False, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.IntegerField(blank=True, null=True)),
                ('updated_by', models.IntegerField(blank=True, null=True)),
                ('department', models.CharField(blank=True, max_length=100, null=True)),
                ('employee_id', models.CharField(blank=True, max_length=50, null=True)),
                ('profile_picture', models.ImageField(blank=True, null=True, upload_to='crm/profile_pictures/')),
                ('bio', models.TextField(blank=True, null=True)),
                ('reporting_manager', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='team_members', to=settings.AUTH_USER_MODEL)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='profile', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'crm_user_profiles',
            },
        ),
        migrations.CreateModel(
            name='UserSettings',
            fields=[
                ('id', models.AutoField(editable=False, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.IntegerField(blank=True, null=True)),
                ('updated_by', models.IntegerField(blank=True, null=True)),
                ('notification_preferences', models.JSONField(default=dict)),
                ('dashboard_layout', models.JSONField(default=dict)),
                ('theme_preference', models.CharField(default='light', max_length=20)),
                ('timezone', models.CharField(default='UTC', max_length=50)),
                ('language', models.CharField(default='en', max_length=10)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='settings', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'crm_user_settings',
            },
        ),
    ]



# Generated by Django 4.2.20 on 2025-06-10 10:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0006_alter_account_options'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='account',
            options={'permissions': [('enable_account_trading', 'Can enable/disable account trading'), ('archive_account', 'Can archive account'), ('reset_account_password', 'Can reset account password'), ('change_account_leverage', 'Can change account leverage'), ('change_account_group', 'Can change account group'), ('manage_account_bonus', 'Can manage account bonus settings'), ('manage_account_markup', 'Can manage account markup settings'), ('change_account_swap_status', 'Can change swap-free status'), ('view_account_trading_activity', 'Can view account trading activity (positions, orders, deals)'), ('view_account_analytics', 'Can view account analytics and charts')], 'verbose_name': 'Account', 'verbose_name_plural': 'Accounts'},
        ),
    ]



from django.db import models

class TransactionDirection(models.TextChoices):
    IN = 'IN', 'Credit/Incoming'
    OUT = 'OUT', 'Debit/Outgoing'

class TransactionType(models.TextChoices):
    DEPOSIT = 'DEPOSIT', 'Deposit'
    WITHDRAWAL = 'WITHDRAWAL', 'Withdrawal'
    BONUS = 'BONUS', 'Bonus'
    REBATE = 'REBATE', 'Rebate'
    COMMISSION = 'COMMISSION', 'Commission'
    CORRECTION = 'CORRECTION', 'Correction'
    INTERNAL_TRANSFER = 'INTERNAL_TRANSFER', 'Internal Transfer'
    CREDIT = 'CREDIT', 'Credit'
    CHARGE = 'CHARGE', 'Charge'
    COMPENSATION = 'COMPENSATION', 'Compensation'

class TransactionStatus(models.TextChoices):
    PENDING = 'PENDING', 'Pending'
    APPROVED = 'APPROVED', 'Approved'
    REJECTED = 'REJECTED', 'Rejected'
    CANCELLED = 'CANCELLED', 'Cancelled'
    FAILED = 'FAILED', 'Failed'
    PROCESSED = 'PROCESSED', 'Processed'

class PaymentMethod(models.TextChoices):
    WIRE = 'WIRE', 'Wire Transfer'
    CRYPTO = 'CRYPTO', 'Cryptocurrency'
    CARD = 'CARD', 'Credit/Debit Card'
    MANUAL = 'MANUAL', 'Manual Entry' 
    SYSTEM = 'SYSTEM', 'System Generated' # For Bonus, Rebate, Internal Transfers etc.
    UPI = 'UPI', 'UPI Payment'
    # Example: Add e-wallets if they are distinct high-level methods
    SKRILL = 'SKRILL', 'Skrill'
    NETELLER = 'NETELLER', 'Neteller'

class PaymentGateway(models.TextChoices):
    STRIPE = 'STRIPE', 'Stripe'
    MYFATOORAH = 'MYFATOORAH', 'MyFatoorah'
    MANUAL = 'MANUAL', 'Manual Processing'
    CRYPTO = 'CRYPTO', 'Crypto Payment'
    WIRE = 'WIRE', 'Wire Transfer'
    SYSTEM = 'SYSTEM', 'System Internal'
    QUBEPAY = 'QUBEPAY', 'QubePay'
    BANK_TRANSFER = 'BANK_TRANSFER', 'Bank Transfer'
    CASH = 'CASH', 'Cash'
    PAYPAL = 'PAYPAL', 'PayPal'
    RAZORPAY = 'RAZORPAY', 'Razorpay'
    UPI = 'UPI', 'UPI'
    
class CardType(models.TextChoices):
    VISA = 'VISA', 'Visa'
    MASTERCARD = 'MASTERCARD', 'Mastercard'
    AMEX = 'AMEX', 'American Express'

class EvidenceType(models.TextChoices):
    BANK_SLIP = 'BANK_SLIP', 'Bank Slip'
    CRYPTO_TX_HASH = 'CRYPTO_TX_HASH', 'Crypto Transaction Hash'
    CARD_RECEIPT = 'CARD_RECEIPT', 'Card Receipt'
    OTHER = 'OTHER', 'Other Document'



# Generated by Django 4.2.21 on 2025-06-28 08:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0015_alter_crmuser_options'),
        ('customers', '0014_alter_customer_options'),
        ('ib_commission', '0006_ibonboardingstatus_creation_source'),
    ]

    operations = [
        migrations.CreateModel(
            name='IBQuestionnaire',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('target_countries', models.JSONField(help_text='List of country codes the IB plans to target')),
                ('expected_clients', models.IntegerField(help_text='Expected number of clients per month')),
                ('have_site', models.CharField(help_text='Whether IB has a website/platform', max_length=255)),
                ('get_client', models.TextField(help_text='How the IB plans to acquire clients')),
                ('ref_other', models.TextField(blank=True, help_text='Previous experience or other referral information')),
                ('source', models.CharField(choices=[('CP', 'Client Portal'), ('CRM', 'CRM')], help_text='Where the questionnaire was submitted from', max_length=10)),
                ('submitted_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('customer', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='ib_questionnaire', to='customers.customer')),
                ('submitted_by', models.ForeignKey(blank=True, help_text='CRM user who submitted on behalf of IB (if source=CRM)', null=True, on_delete=django.db.models.deletion.SET_NULL, to='users.crmuser')),
            ],
            options={
                'verbose_name': 'IB Questionnaire',
                'verbose_name_plural': 'IB Questionnaires',
                'db_table': 'ib_questionnaire',
            },
        ),
    ]



# Generated by Django 5.1.4 on 2025-02-04 11:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='account',
            name='group_type',
        ),
        migrations.AddField(
            model_name='account',
            name='has_bonus',
            field=models.BooleanField(default=False, help_text='Whether this account has bonus applied'),
        ),
        migrations.AddField(
            model_name='account',
            name='has_markup',
            field=models.BooleanField(default=False, help_text='Whether this account has markup applied'),
        ),
    ]
